@@ -1,13 +1,16 @@
 from argparse import ArgumentParser
 from typing import List, Optional
 
+import matplotlib.style
 import numpy as np
 
 import euphonic
-from euphonic import ureg, Spectrum2D
+from euphonic import ureg, Spectrum2D, QpointFrequencies, ForceConstants
 import euphonic.plot
 from euphonic.util import get_qpoint_labels
+from euphonic.styles import base_style
 from .utils import (_bands_from_force_constants, _calc_modes_kwargs,
+                    _compose_style, _plot_label_kwargs,
                     get_args, _get_debye_waller,
                     _get_energy_bins, _get_q_distance,
                     _get_cli_parser, load_data_from_file,
@@ -18,21 +21,28 @@ def main(params: Optional[List[str]] = None) -> None:
     args = get_args(get_parser(), params)
     calc_modes_kwargs = _calc_modes_kwargs(args)
 
-    data = load_data_from_file(args.filename)
+    frequencies_only = (args.weighting != 'coherent')
+    data = load_data_from_file(args.filename, verbose=True,
+                               frequencies_only=frequencies_only)
+    if not frequencies_only and type(data) is QpointFrequencies:
+        raise TypeError('Eigenvectors are required to use '
+                        '"--weighting coherent" option')
+    if args.weighting.lower() == 'coherent' and args.temperature is not None:
+        if not isinstance(data, ForceConstants):
+            raise TypeError('Force constants data is required to generate '
+                            'the Debye-Waller factor. Leave "--temperature" '
+                            'unset if plotting precalculated phonon modes.')
 
     q_spacing = _get_q_distance(args.length_unit, args.q_spacing)
     recip_length_unit = q_spacing.units
 
-    frequencies_only = (args.weighting != 'coherent')
-
-    if isinstance(data, euphonic.ForceConstants):
-        print("Force Constants data was loaded. Getting band path...")
+    if isinstance(data, ForceConstants):
+        print("Getting band path...")
         (modes, x_tick_labels, split_args) = _bands_from_force_constants(
             data, q_distance=q_spacing, insert_gamma=False,
             frequencies_only=frequencies_only,
             **calc_modes_kwargs)
-    elif isinstance(data, euphonic.QpointPhononModes):
-        print("Phonon band data was loaded.")
+    else:
         modes = data
         split_args = {'btol': args.btol}
         x_tick_labels = get_qpoint_labels(modes.qpts,
@@ -45,12 +55,6 @@ def main(params: Optional[List[str]] = None) -> None:
 
     if args.weighting.lower() == 'coherent':
         if args.temperature is not None:
-            if not isinstance(data, euphonic.ForceConstants):
-                raise TypeError("Cannot generate Debye-Waller factor without "
-                                "force constants data. Leave --temperature "
-                                "unset if plotting precalculated phonon "
-                                "modes.")
-
             temperature = args.temperature * ureg('K')
             dw = _get_debye_waller(temperature, data,
                                    grid=args.grid,
@@ -72,17 +76,11 @@ def main(params: Optional[List[str]] = None) -> None:
                      if args.q_broadening else None),
             y_width=(args.energy_broadening * ebins.units
                      if args.energy_broadening else None),
-            shape=args.shape)
+            shape=args.shape, method='convolve')
 
     print("Plotting figure")
-    if args.y_label is None:
-        y_label = f"Energy / {spectrum.y_data.units:~P}"
-    else:
-        y_label = args.y_label
-    if args.x_label is None:
-        x_label = ""
-    else:
-        x_label = args.x_label
+    plot_label_kwargs = _plot_label_kwargs(
+        args, default_ylabel=f"Energy / {spectrum.y_data.units:~P}")
 
     if x_tick_labels:
         spectrum.x_tick_labels = x_tick_labels
@@ -91,13 +89,16 @@ def main(params: Optional[List[str]] = None) -> None:
     if len(spectra) > 1:
         print(f"Found {len(spectra)} regions in q-point path")
 
-    euphonic.plot.plot_2d(spectra,
-                          cmap=args.cmap,
-                          vmin=args.v_min, vmax=args.v_max,
-                          x_label=x_label,
-                          y_label=y_label,
-                          title=args.title)
-    matplotlib_save_or_show(save_filename=args.save_to)
+    if args.save_json:
+        spectrum.to_json_file(args.save_json)
+    style = _compose_style(user_args=args, base=[base_style])
+    with matplotlib.style.context(style):
+
+        euphonic.plot.plot_2d(spectra,
+                              vmin=args.vmin,
+                              vmax=args.vmax,
+                              **plot_label_kwargs)
+        matplotlib_save_or_show(save_filename=args.save_to)
 
 
 def get_parser() -> ArgumentParser:
