@@ -3,9 +3,11 @@ import collections
 import copy
 import itertools
 import math
+import json
 from numbers import Integral
 from typing import (Any, Dict, List, Optional, overload,
-                    Sequence, Tuple, TypeVar, Union)
+                    Sequence, Tuple, TypeVar, Union, Type, Iterable)
+import warnings
 
 import numpy as np
 from scipy.ndimage import correlate1d, gaussian_filter
@@ -15,16 +17,12 @@ from euphonic.io import (_obj_to_json_file, _obj_from_json_file,
                          _obj_to_dict, _process_dict)
 from euphonic.readers.castep import read_phonon_dos_data
 from euphonic.util import _get_unique_elems_and_idx
-from euphonic import ureg, Quantity
-
-
-S = TypeVar('S', bound='Spectrum')
-S1D = TypeVar('S1D', bound='Spectrum1D')
-SC = TypeVar('SC', bound='Spectrum1DCollection')
-S2D = TypeVar('S2D', bound='Spectrum2D')
+from euphonic import ureg, Quantity, __version__
 
 
 class Spectrum(ABC):
+    T = TypeVar('T', bound='Spectrum')
+
     def __setattr__(self, name: str, value: Any) -> None:
         _check_unit_conversion(self, name, value,
                                ['x_data_unit', 'y_data_unit'])
@@ -37,25 +35,49 @@ class Spectrum(ABC):
         setattr(self, f'{attr_name}_data_unit', str(data.units))
 
     @property
-    def x_data(self):
+    def x_data(self) -> Quantity:
         return self._x_data*ureg(self._internal_x_data_unit).to(
             self.x_data_unit)
 
     @x_data.setter
-    def x_data(self, value):
+    def x_data(self, value: Quantity) -> None:
         self.x_data_unit = str(value.units)
         self._x_data = value.to(self._internal_x_data_unit).magnitude
 
     @property
-    def y_data(self):
+    def y_data(self) -> Quantity:
         return self._y_data*ureg(self._internal_y_data_unit).to(
             self.y_data_unit)
 
     @y_data.setter
-    def y_data(self, value):
+    def y_data(self, value: Quantity) -> None:
         self.y_data_unit = str(value.units)
         self._y_data = value.to(self._internal_y_data_unit).magnitude
 
+    @property
+    def x_tick_labels(self) -> List[Tuple[int, str]]:
+        return self._x_tick_labels
+
+    @x_tick_labels.setter
+    def x_tick_labels(self, value: Sequence[Tuple[int, str]]) -> None:
+        err_msg = ('x_tick_labels should be of type '
+                   'Sequence[Tuple[int, str]] e.g. '
+                   '[(0, "label1"), (5, "label2")]')
+        if value is not None:
+            if isinstance(value, Sequence):
+                for elem in value:
+                    if not (isinstance(elem, tuple)
+                            and len(elem) == 2
+                            and isinstance(elem[0], Integral)
+                            and isinstance(elem[1], str)):
+                        raise TypeError(err_msg)
+                # Ensure indices in x_tick_labels are plain ints as
+                # np.int64/32 etc. are not JSON serializable
+                value = [(int(idx), label) for idx, label in value]
+            else:
+                raise TypeError(err_msg)
+        self._x_tick_labels = value
+
     @abstractmethod
     def to_dict(self) -> Dict[str, Any]:
         """Write to dict using euphonic.io._obj_to_dict"""
@@ -63,7 +85,7 @@ class Spectrum(ABC):
 
     @classmethod
     @abstractmethod
-    def from_dict(cls: S, d: Dict[str, Any]) -> S:
+    def from_dict(cls: Type[T], d: Dict[str, Any]) -> T:
         """Initialise a Spectrum object from dictionary"""
         ...
 
@@ -80,7 +102,7 @@ class Spectrum(ABC):
         _obj_to_json_file(self, filename)
 
     @classmethod
-    def from_json_file(cls: S, filename: str) -> S:
+    def from_json_file(cls: Type[T], filename: str) -> T:
         """
         Read from a JSON file. See from_dict for required fields
 
@@ -93,12 +115,12 @@ class Spectrum(ABC):
         return _obj_from_json_file(cls, filename, type_dict)
 
     @abstractmethod
-    def _split_by_indices(self: S, indices: Union[Sequence[int], np.ndarray]
-                          ) -> List[S]:
+    def _split_by_indices(self: T, indices: Union[Sequence[int], np.ndarray]
+                          ) -> List[T]:
         """Split data along x axis at given indices"""
         ...
 
-    def _split_by_tol(self: S, btol: float = 10.0) -> List[S]:
+    def _split_by_tol(self: T, btol: float = 10.0) -> List[T]:
         """Split data along x-axis at detected breakpoints"""
         diff = np.diff(self.x_data.magnitude)
         median = np.median(diff)
@@ -107,7 +129,7 @@ class Spectrum(ABC):
 
     @staticmethod
     def _ranges_from_indices(indices: Union[Sequence[int], np.ndarray]
-                             ) -> List[Tuple[int, int]]:
+                             ) -> List[Tuple[int, Optional[int]]]:
         """Convert a series of breakpoints to a series of slice ranges"""
         if len(indices) == 0:
             ranges = [(0, None)]
@@ -120,9 +142,10 @@ class Spectrum(ABC):
         return ranges
 
     @staticmethod
-    def _cut_x_ticks(x_tick_labels: Union[List[Tuple[int, str]], None],
+    def _cut_x_ticks(x_tick_labels: Union[Sequence[Tuple[int, str]], None],
                      x0: int,
-                     x1: Union[int, None]) -> List[Tuple[int, str]]:
+                     x1: Union[int, None]) -> Union[List[Tuple[int, str]],
+                                                    None]:
         """Crop and shift x labels to new x range"""
         if x_tick_labels is None:
             return None
@@ -130,8 +153,8 @@ class Spectrum(ABC):
             return [(int(x - x0), label) for (x, label) in x_tick_labels
                     if (x >= x0) and ((x1 is None) or (x < x1))]
 
-    def split(self: S, indices: Union[Sequence[int], np.ndarray] = None,
-              btol: float = None) -> List[S]:
+    def split(self: T, indices: Union[Sequence[int], np.ndarray] = None,
+              btol: float = None) -> List[T]:
         """Split to multiple spectra
 
         Data may be split by index. Alternatively, x-axis data may be
@@ -141,9 +164,9 @@ class Spectrum(ABC):
 
         Parameters
         ----------
-        indices:
+        indices
             positions in data of breakpoints
-        btol:
+        btol
             parameter used to identify breakpoints. This is a ratio
             between the gap in values and the median gap between
             neighbouring x-values. If neither indices nor btol is
@@ -151,10 +174,10 @@ class Spectrum(ABC):
 
         Returns
         -------
+        split_spectra
             Separated spectrum regions. If passed to the appropriate
             functions in euphonic.plot this would be interpreted as a
             series of subplots.
-
         """
 
         if indices is None:
@@ -168,7 +191,8 @@ class Spectrum(ABC):
 
     @staticmethod
     def _broaden_data(data: np.ndarray, bin_centres: Sequence[np.ndarray],
-                      widths: Sequence[float], shape: str = 'gauss'
+                      widths: Sequence[float], shape: str = 'gauss',
+                      method: Optional[str] = None
                       ) -> np.ndarray:
         """
         Broaden data along each axis by widths, returning the broadened
@@ -176,6 +200,39 @@ class Spectrum(ABC):
         widths should match the number of dimensions in data.
         bin_centres and widths are assumed to be in the same units
         """
+        shape_opts = ('gauss', 'lorentz')
+        if shape not in shape_opts:
+            raise ValueError(f'Invalid value for shape, got {shape}, '
+                             f'should be one of {shape_opts}')
+        method_opts = ('convolve', None)
+        if method not in method_opts:
+            raise ValueError(f'Invalid value for method, got {method}, '
+                             f'should be one of {method_opts}')
+
+        # We only want to check for unequal bins if using a method that
+        # is not correct for unequal bins (currently this is the only
+        # option but leave the 'if' anyway for future implementations)
+        if method in ('convolve', None):
+            axes = ['x_data', 'y_data']
+            unequal_bin_axes = []
+            for ax, (width, bin_data) in enumerate(zip(widths, bin_centres)):
+                if width is not None:
+                    bin_widths = np.diff(bin_data)
+                    if not np.all(np.isclose(bin_widths, bin_widths[0])):
+                        unequal_bin_axes += [axes[ax]]
+            if len(unequal_bin_axes) > 0:
+                msg = (f'{" and ".join(unequal_bin_axes)} bin widths are '
+                       f'not equal, so broadening by convolution may give '
+                       f'incorrect results.')
+                if method is None:
+                    warnings.warn(
+                        msg + ' In the future, this will raise a ValueError, '
+                        'so if you still want to broaden by convolution '
+                        'please explicitly use the method="convolve" option.',
+                        category=DeprecationWarning, stacklevel=3)
+                else:
+                    warnings.warn(msg, stacklevel=3)
+
         if shape == 'gauss':
             sigmas = []
             for width, bin_data in zip(widths, bin_centres):
@@ -191,9 +248,6 @@ class Spectrum(ABC):
                     broadening = _distribution_1d(bin_data, width)
                     data_broadened = correlate1d(data_broadened, broadening,
                                                  mode='constant', axis=ax)
-        else:
-            raise ValueError(
-                f"Distribution shape '{shape}' not recognised")
         return data_broadened
 
     @staticmethod
@@ -291,15 +345,19 @@ class Spectrum1D(Spectrum):
         == (n_x_data + 1,))
     y_data
         Shape (n_x_data,) float Quantity. The plot data in y
-    x_tick_labels : Sequence[Tuple[int, str]] or None
-        Special tick labels e.g. for high-symmetry points. The int
-        refers to the index in x_data the label should be applied to
-    metadata : Dict[str, Union[int, str]]
-        Contains metadata about the spectrum. Keys should be strings
-        and values should be strings or integers
+    x_tick_labels
+        Sequence[Tuple[int, str]] or None. Special tick labels e.g. for
+        high-symmetry points. The int refers to the index in x_data the
+        label should be applied to
+    metadata
+        Dict[str, Union[int, str]]. Contains metadata about the
+        spectrum. Keys should be strings and values should be strings
+        or integers
         There are some functional keys:
           - 'label' : str. This is used label lines on a 1D plot
     """
+    T = TypeVar('T', bound='Spectrum1D')
+
     def __init__(self, x_data: Quantity, y_data: Quantity,
                  x_tick_labels: Optional[Sequence[Tuple[int, str]]] = None,
                  metadata: Optional[Dict[str, Union[int, str]]] = None
@@ -336,7 +394,7 @@ class Spectrum1D(Spectrum):
         self.x_tick_labels = x_tick_labels
         self.metadata = {} if metadata is None else metadata
 
-    def __add__(self, other: S1D):
+    def __add__(self, other: 'Spectrum1D') -> 'Spectrum1D':
         """
         Sums the y_data of two Spectrum1D objects together,
         their x_data axes must be equal, and their y_data must
@@ -349,9 +407,9 @@ class Spectrum1D(Spectrum):
         spec_col = Spectrum1DCollection.from_spectra([self, other])
         return spec_col.sum()
 
-    def _split_by_indices(self: S1D,
+    def _split_by_indices(self: T,
                           indices: Union[Sequence[int], np.ndarray]
-                          ) -> List[S1D]:
+                          ) -> List[T]:
         """Split data along x-axis at given indices"""
         ranges = self._ranges_from_indices(indices)
 
@@ -373,8 +431,27 @@ class Spectrum1D(Spectrum):
         return _obj_to_dict(self, ['x_data', 'y_data', 'x_tick_labels',
                                    'metadata'])
 
+    def to_text_file(self, filename: str,
+                     fmt: Optional[Union[str, Sequence[str]]] = None) -> None:
+        """
+        Write to a text file. The header contains metadata and unit
+        information, the first column contains x_data and the second
+        column contains y_data. Note that text files written in this
+        format cannot be read back in by Euphonic.
+
+        Parameters
+        ----------
+        filename
+            Name of the text file to write to
+        fmt
+            A format specifier or sequence of specifiers (one for each
+            column), to be passed to numpy.savetxt
+        """
+        spec = Spectrum1DCollection.from_spectra([self])
+        spec.to_text_file(filename, fmt)
+
     @classmethod
-    def from_dict(cls: S1D, d: Dict[str, Any]) -> S1D:
+    def from_dict(cls: Type[T], d: Dict[str, Any]) -> T:
         """
         Convert a dictionary to a Spectrum1D object
 
@@ -403,8 +480,8 @@ class Spectrum1D(Spectrum):
                    metadata=d['metadata'])
 
     @classmethod
-    def from_castep_phonon_dos(cls: S1D, filename: str,
-                               element: Optional[str] = None) -> S1D:
+    def from_castep_phonon_dos(cls: Type[T], filename: str,
+                               element: Optional[str] = None) -> T:
         """
         Reads DOS from a CASTEP .phonon_dos file
 
@@ -423,11 +500,12 @@ class Spectrum1D(Spectrum):
         else:
             metadata['species'] = element
         metadata['label'] = element
-        return Spectrum1D(data['dos_bins']*ureg(data['dos_bins_unit']),
-                          data['dos'][element]*ureg(data['dos_unit']),
-                          metadata=metadata)
+        return cls(data['dos_bins']*ureg(data['dos_bins_unit']),
+                   data['dos'][element]*ureg(data['dos_unit']),
+                   metadata=metadata)
 
-    def broaden(self: S1D, x_width: Quantity, shape: str = 'gauss') -> S1D:
+    def broaden(self: T, x_width: Quantity, shape: str = 'gauss',
+                method: Optional[str] = None) -> T:
         """
         Broaden y_data and return a new broadened spectrum object
 
@@ -437,6 +515,12 @@ class Spectrum1D(Spectrum):
             Scalar float Quantity. The broadening FWHM
         shape
             One of {'gauss', 'lorentz'}. The broadening shape
+        method
+            Can be None or 'convolve'. Currently the only broadening
+            method available is convolution with a broadening kernel,
+            but this may not produce correct results for unequal bin
+            widths. To use convolution anyway, explicitly set
+            method='convolve'
 
         Returns
         -------
@@ -447,13 +531,15 @@ class Spectrum1D(Spectrum):
         ------
         ValueError
             If shape is not one of the allowed strings
+        ValueError
+            If method is None and bins are not of equal size
         """
         y_broadened = self._broaden_data(
             self.y_data.magnitude,
             [self.get_bin_centres().magnitude],
             [x_width.to(self.x_data_unit).magnitude],
-            shape=shape)
-        return Spectrum1D(
+            shape=shape, method=method)
+        return type(self)(
             np.copy(self.x_data.magnitude)*ureg(self.x_data_unit),
             y_broadened*ureg(self.y_data_unit),
             copy.copy((self.x_tick_labels)),
@@ -479,19 +565,23 @@ class Spectrum1DCollection(collections.abc.Sequence, Spectrum):
     y_data
         Shape (n_entries, n_x_data) float Quantity. The plot data in y,
         in rows corresponding to separate 1D spectra
-    x_tick_labels : Sequence[Tuple[int, str]] or None
-        Special tick labels e.g. for high-symmetry points. The int
-        refers to the index in x_data the label should be applied to
-    metadata : Dict[str, Union[int, str, LineData]] or None
-        Contains metadata about the spectra. Keys should be strings
-        and values should be strings or integers. There are some
-        functional keys:
+    x_tick_labels
+        Sequence[Tuple[int, str]] or None. Special tick labels e.g. for
+        high-symmetry points. The int refers to the index in x_data the
+        label should be applied to
+    metadata
+        Dict[str, Union[int, str, LineData]] or None. Contains metadata
+        about the spectra. Keys should be strings and values should be
+        strings or integers.
+        There are some functional keys:
           - 'line_data' : LineData
                           This is a Sequence[Dict[str, Union[int, str]],
                           it contains metadata for each spectrum in
                           the collection, and must be of length
                           n_entries
     """
+    T = TypeVar('T', bound='Spectrum1DCollection')
+
     def __init__(
             self, x_data: Quantity, y_data: Quantity,
             x_tick_labels: Optional[Sequence[Tuple[int, str]]] = None,
@@ -542,7 +632,7 @@ class Spectrum1DCollection(collections.abc.Sequence, Spectrum):
                     f'{len(metadata["line_data"])} entries')
         self.metadata = {} if metadata is None else metadata
 
-    def __add__(self, other: SC):
+    def __add__(self: T, other: T) -> T:
         """
         Appends the y_data of 2 Spectrum1DCollection objects,
         creating a single Spectrum1DCollection that contains
@@ -555,11 +645,11 @@ class Spectrum1DCollection(collections.abc.Sequence, Spectrum):
         spectra are retained in the top level dictionary, any
         others are put in the individual 'line_data' entries
         """
-        return Spectrum1DCollection.from_spectra([*self, *other])
+        return type(self).from_spectra([*self, *other])
 
-    def _split_by_indices(self: SC,
+    def _split_by_indices(self,
                           indices: Union[Sequence[int], np.ndarray]
-                          ) -> List[SC]:
+                          ) -> List[T]:
         """Split data along x-axis at given indices"""
 
         ranges = self._ranges_from_indices(indices)
@@ -578,14 +668,14 @@ class Spectrum1DCollection(collections.abc.Sequence, Spectrum):
         ...
 
     @overload  # noqa: F811
-    def __getitem__(self, item: slice) -> SC:
+    def __getitem__(self, item: slice) -> T:
         ...
 
     @overload  # noqa: F811
-    def __getitem__(self, item: Union[Sequence[int], np.ndarray]) -> SC:
+    def __getitem__(self, item: Union[Sequence[int], np.ndarray]) -> T:
         ...
 
-    def __getitem__(self, item: Union[int, slice, Sequence[int]]):  # noqa: F811
+    def __getitem__(self, item: Union[int, slice, Sequence[int], np.ndarray]):  # noqa: F811
         new_metadata = copy.deepcopy(self.metadata)
         line_metadata = new_metadata.pop('line_data',
                                          [{} for _ in self._y_data])
@@ -616,7 +706,7 @@ class Spectrum1DCollection(collections.abc.Sequence, Spectrum):
                           metadata=new_metadata)
 
     @classmethod
-    def from_spectra(cls: SC, spectra: Sequence[Spectrum1D]) -> SC:
+    def from_spectra(cls: Type[T], spectra: Sequence[Spectrum1D]) -> T:
         if len(spectra) < 1:
             raise IndexError("At least one spectrum is needed for collection")
 
@@ -648,7 +738,7 @@ class Spectrum1DCollection(collections.abc.Sequence, Spectrum):
 
     @staticmethod
     def _combine_metadata(all_metadata: Sequence[Dict[str, Union[int, str]]]
-                          ) -> Dict[str, Any]:
+                          ) -> Dict[str, Union[int, str, LineData]]:
         """
         From a sequence of metadata dictionaries, combines all common
         key/value pairs into the top level of a metadata dictionary,
@@ -728,8 +818,41 @@ class Spectrum1DCollection(collections.abc.Sequence, Spectrum):
         return _obj_to_dict(self, ['x_data', 'y_data', 'x_tick_labels',
                                    'metadata'])
 
+    def to_text_file(self, filename: str,
+                     fmt: Optional[Union[str, Sequence[str]]] = None) -> None:
+        """
+        Write to a text file. The header contains metadata and unit
+        information, the first column is x_data and each subsequent
+        column is a y_data spectrum. Note that text files written in
+        this format cannot be read back in by Euphonic.
+
+        Parameters
+        ----------
+        filename
+            Name of the text file to write to
+        fmt
+            A format specifier or sequence of specifiers (one for each
+            column), to be passed to numpy.savetxt
+        """
+        common_metadata = copy.deepcopy(self.metadata)
+        line_data = common_metadata.pop('line_data',
+                                        [{} for i in self.y_data])
+        header = [f'Generated by Euphonic {__version__}',
+                  f'x_data in ({self.x_data.units})',
+                  f'y_data in ({self.y_data.units})',
+                  f'Common metadata: {json.dumps(common_metadata)}',
+                  f'Column 1: x_data']
+        for i, line in enumerate(line_data):
+            header += [f'Column {i + 2}: y_data[{i}] {json.dumps(line)}']
+        out_data = np.hstack((self.get_bin_centres().magnitude[:, np.newaxis],
+                              self.y_data.transpose().magnitude))
+        kwargs = {'header': '\n'.join(header)}
+        if fmt is not None:
+            kwargs['fmt'] = fmt
+        np.savetxt(filename, out_data, **kwargs)
+
     @classmethod
-    def from_dict(cls: SC, d) -> SC:
+    def from_dict(cls: Type[T], d) -> T:
         """
         Convert a dictionary to a Spectrum1DCollection object
 
@@ -758,7 +881,7 @@ class Spectrum1DCollection(collections.abc.Sequence, Spectrum):
                    metadata=d['metadata'])
 
     @classmethod
-    def from_castep_phonon_dos(cls: SC, filename: str) -> SC:
+    def from_castep_phonon_dos(cls: Type[T], filename: str) -> T:
         """
         Reads total DOS and per-element PDOS from a CASTEP
         .phonon_dos file
@@ -783,7 +906,8 @@ class Spectrum1DCollection(collections.abc.Sequence, Spectrum):
             y_data*ureg(data['dos_unit']),
             metadata=metadata)
 
-    def broaden(self, x_width: Quantity, shape: str = 'gauss') -> SC:
+    def broaden(self, x_width: Quantity, shape: str = 'gauss',
+                method: Optional[str] = None) -> T:
         """
         Individually broaden each line in y_data, returning a new
         Spectrum1DCollection
@@ -794,6 +918,12 @@ class Spectrum1DCollection(collections.abc.Sequence, Spectrum):
             Scalar float Quantity. The broadening FWHM
         shape
             One of {'gauss', 'lorentz'}. The broadening shape
+        method
+            Can be None or 'convolve'. Currently the only broadening
+            method available is convolution with a broadening kernel,
+            but this may not produce correct results for unequal bin
+            widths. To use convolution anyway, explicitly set
+            method='convolve'
 
         Returns
         -------
@@ -804,20 +934,23 @@ class Spectrum1DCollection(collections.abc.Sequence, Spectrum):
         ------
         ValueError
             If shape is not one of the allowed strings
+        ValueError
+            If method is None and bins are not of equal size
         """
         y_broadened = np.zeros_like(self.y_data)
         x_centres = [self.get_bin_centres().magnitude]
         x_width_calc = [x_width.to(self.x_data_unit).magnitude]
         for i, yi in enumerate(self.y_data.magnitude):
             y_broadened[i] = self._broaden_data(
-                yi, x_centres, x_width_calc, shape=shape)
+                yi, x_centres, x_width_calc, shape=shape,
+                method=method)
         return Spectrum1DCollection(
             np.copy(self.x_data.magnitude)*ureg(self.x_data_unit),
             y_broadened*ureg(self.y_data_unit),
             copy.copy((self.x_tick_labels)),
             copy.deepcopy(self.metadata))
 
-    def group_by(self, *line_data_keys: str) -> SC:
+    def group_by(self, *line_data_keys: str) -> T:
         """
         Group and sum y_data for each spectrum according to the values
         mapped to the specified keys in metadata['line_data']
@@ -878,7 +1011,7 @@ class Spectrum1DCollection(collections.abc.Sequence, Spectrum):
                           metadata=metadata)
 
     def select(self, **select_key_values: Union[
-            str, int, Sequence[str], Sequence[int]]) -> SC:
+            str, int, Sequence[str], Sequence[int]]) -> T:
         """
         Select spectra by their keys and values in metadata['line_data']
 
@@ -941,20 +1074,24 @@ class Spectrum2D(Spectrum):
         == (n_y_data + 1,))
     z_data
         Shape (n_x_data, n_y_data) float Quantity. The plot data in z
-    x_tick_labels : Sequence[Tuple[int, str]] or None
-        Special tick labels e.g. for high-symmetry points. The int
-        refers to the index in x_data the label should be applied to
-    metadata : Dict[str, Union[int, str]]
-        Contains metadata about the spectrum. Keys should be strings
-        and values should be strings or integers
+    x_tick_labels
+        Sequence[Tuple[int, str]] or None. Special tick labels e.g. for
+        high-symmetry points. The int refers to the index in x_data the
+        label should be applied to
+    metadata
+        Dict[str, Union[int, str]]. Contains metadata about the
+        spectrum. Keys should be strings and values should be strings
+        or integers
     """
+    T = TypeVar('T', bound='Spectrum2D')
+
     def __init__(self, x_data: Quantity, y_data: Quantity,
                  z_data: Quantity,
                  x_tick_labels: Optional[Sequence[Tuple[int, str]]] = None,
                  metadata: Optional[Dict[str, Union[int, str]]] = None
                  ) -> None:
         """
-        Attributes
+        Parameters
         ----------
         x_data
             Shape (n_x_data,) or (n_x_data + 1,) float Quantity. The
@@ -992,12 +1129,12 @@ class Spectrum2D(Spectrum):
         self.metadata = {} if metadata is None else metadata
 
     @property
-    def z_data(self):
+    def z_data(self) -> Quantity:
         return self._z_data*ureg(self._internal_z_data_unit).to(
             self.z_data_unit)
 
     @z_data.setter
-    def z_data(self, value):
+    def z_data(self, value: Quantity) -> None:
         self.z_data_unit = str(value.units)
         self._z_data = value.to(self._internal_z_data_unit).magnitude
 
@@ -1006,9 +1143,9 @@ class Spectrum2D(Spectrum):
                                ['z_data_unit'])
         super(Spectrum2D, self).__setattr__(name, value)
 
-    def _split_by_indices(self: S2D,
+    def _split_by_indices(self,
                           indices: Union[Sequence[int], np.ndarray]
-                          ) -> List[S2D]:
+                          ) -> List[T]:
         """Split data along x-axis at given indices"""
         ranges = self._ranges_from_indices(indices)
         return [type(self)(self.x_data[x0:x1], self.y_data,
@@ -1018,9 +1155,9 @@ class Spectrum2D(Spectrum):
                            metadata=self.metadata)
                 for x0, x1 in ranges]
 
-    def broaden(self: S2D, x_width: Optional[Quantity] = None,
-                y_width: Optional[Quantity] = None, shape: str = 'gauss'
-                ) -> S2D:
+    def broaden(self, x_width: Optional[Quantity] = None,
+                y_width: Optional[Quantity] = None, shape: str = 'gauss',
+                method: Optional[str] = None) -> T:
         """
         Broaden z_data and return a new broadened Spectrum2D object
 
@@ -1032,6 +1169,12 @@ class Spectrum2D(Spectrum):
             Scalar float Quantity. The broadening FWHM in y
         shape
             One of {'gauss', 'lorentz'}. The broadening shape
+        method
+            Can be None or 'convolve'. Currently the only broadening
+            method available is convolution with a broadening kernel,
+            but this may not produce correct results for unequal bin
+            widths. To use convolution anyway, explicitly set
+            method='convolve'
 
         Returns
         -------
@@ -1042,6 +1185,8 @@ class Spectrum2D(Spectrum):
         ------
         ValueError
             If shape is not one of the allowed strings
+        ValueError
+            If method is None and bins are not of equal size
         """
         bin_centres = [self.get_bin_centres(ax).magnitude for ax in ['x', 'y']]
         bin_widths = [None]*2
@@ -1050,7 +1195,8 @@ class Spectrum2D(Spectrum):
         if y_width is not None:
             bin_widths[1] = y_width.to(self.y_data_unit).magnitude
         z_broadened = self._broaden_data(self.z_data.magnitude, bin_centres,
-                                         bin_widths, shape=shape)
+                                         bin_widths, shape=shape,
+                                         method=method)
         return Spectrum2D(
             np.copy(self.x_data.magnitude)*ureg(self.x_data_unit),
             np.copy(self.y_data.magnitude)*ureg(self.y_data_unit),
@@ -1120,7 +1266,7 @@ class Spectrum2D(Spectrum):
                                    'x_tick_labels', 'metadata'])
 
     @classmethod
-    def from_dict(cls: S2D, d: Dict[str, Any]) -> S2D:
+    def from_dict(cls: Type[T], d: Dict[str, Any]) -> T:
         """
         Convert a dictionary to a Spectrum2D object
 
